from unittest.mock import patch, PropertyMock
from reddit_view import view

import unittest
import json


class TestRedditLogic(unittest.TestCase):

    def setUp(self):
        self.initial = {
                'i': 'r',
                's': 'pics,',
                'o': 'hot,new,top',
                'p': '100',
                'c': '100'}

    def test_kwargs_in_self(self):
        reddit = view.RedditLogic(**self.initial)
        for attribute in self.initial:
            self.assertIn(attribute, dir(reddit))
        with patch.dict(self.initial, {'test1': 'value'}, clear=True):
            fake = view.RedditLogic(**self.initial)
            for attribute in self.initial:
                self.assertIn(attribute, dir(fake))

    @patch('reddit_view.view.RedditLogic.get_json')
    def test_dispatch(self, MockReddit):
        url = 'it works.jpg'
        data = {'data':
                {'children':
                    [
                        {'data': {'score': 9000, 'url': url, }, },
                    ], }, }

        MockReddit.return_value = data
        reddit = view.RedditLogic(**self.initial)
        assert MockReddit is view.RedditLogic.get_json
        assert MockReddit.called_called_with(**self.initial)
        test_value = reddit.dispatch()
        self.assertEqual(test_value, set([url, ]))

    def test_match_pattern(self):
        good = [
                '/this.jpg']
        bad = [
                '/comments/',
                '/a/',
                '/gallery/',
                '/video/',
                '/html/',
                '/', ]

        value = sorted(good + bad)

        with patch(
                'reddit_view.view.RedditLogic.urls_to_filter',
                new_callable=PropertyMock) \
                as mock_foo:

            mock_foo.return_value = value
            reddit = view.RedditLogic(**self.initial)
            self.assertEqual(reddit.urls_to_filter, value)
            clean = reddit.match_pattern()
            self.assertNotIn(bad, clean)

    @patch('reddit_view.view.RedditLogic.get_json')
    def test_extract_image_url(self, MockReddit):
        return True
        # TODO test gets url
        url = 'it works.jpg'
        data = {
              "data": {
                "children": {
                  "data": {
                    "url": url,
                    "score": 5,
                  }
                },
              }
            }

        import json
        data = json.dumps(data)

        MockReddit.return_value = data
        reddit = view.RedditLogic(**self.initial)
        test_value = reddit.dispatch()
        self.assertEqual(test_value, [ url ])

    @patch('reddit_view.view.requests.get')
    def test_get_json(self, MockRequests):
        mock_json = unittest.mock.Mock(side_effect=KeyError('foo'))
        MockRequests.return_value = mock_json
        reddit = view.RedditLogic(**self.initial)
        reddit.get_json('http://test')
        self.assertTrue(mock_json.json.called)


class TestImgurGallery(unittest.TestCase):

    def setUp(self):
        self.path = 'google.com'

    @patch('view.requests.get')
    def test_gen_list(self, mock_get):
        mock_get.side_effect = [ unittest.mock.MagicMock(status_code=200, content='<html></html>') ]

        img_class = view.ImgurGallery(**{'url': self.path})
        value = img_class.dispatch()
        self.assertEqual(img_class.page.content, '<html></html>')
        self.assertEqual(value, [])



if __name__ == '__main__':
        unittest.main()



from .view import RedditLogic

import argparse


def set_up_parser():
    parser = argparse.ArgumentParser(description='Parse arguments')
    parser.add_argument(
            '-i', '--i',
            metavar='r', nargs='?',
            help='Index for subs', default='r', required=False)

    parser.add_argument(
            '-s', '--s',
            metavar='funy,', nargs='?',
            help='comma seperated list of subreddits',
            default='funy,', required=False)

    parser.add_argument(
            '-o', '--o',
            metavar='hot,top', nargs='?',
            help='comma seperated list of orders',
            default='hot,', required=False)

    parser.add_argument(
            '-c', '--c',
            metavar='nnn', nargs='?',
            help='count of posts to consider',
            default='100', required=False)

    parser.add_argument(
            '-p', '--p',
            metavar='nnn', nargs='?',
            help='count of points post must have',
            default='100', required=False)

    args = parser.parse_args()
    return args


def main():
    # return args
    args = set_up_parser()
    reddit = RedditLogic(**vars(args))
    show = reddit.dispatch()
    print(*show, sep='\n')

if __name__ == "__main__":
    main()



#!/usr/bin/env python
import sys, requests, json, re, itertools, time, pprint

from bs4 import BeautifulSoup
from fake_useragent import UserAgent

def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)

PATTERNS = {
    'gallery':  (lambda x: re.compile(".+\/(a|gallery|gfycat)\/.+$").match(x)),
    'comments': (lambda x: re.compile(".+\/comments\/.+$").match(x)),
    'ext':  (lambda x: re.compile("^.+\.(jpg|png|jpeg|gif)(.+)?$").match(x)),
    'imgur':  (lambda x: re.compile(".+\/imgur\/.+$").match(x)),
    'not_img': (lambda x: re.compile(".+(video|html|\/)(.+)?$").match(x)),
    }

ua = UserAgent()
request_headers = {
        'User-Agent': ua.random,
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
        'Accept-Language': 'en-US,en;q=0.5',
        'Connection': 'keep-alive',
        'Cache-Control': 'max-age=0',
        }


class RedditLogic:

    combinations = []
    urls = []
    urls_to_filter = []

    def __init__(self, *args, **kwargs):
        self.__dict__.update(kwargs)

    def dispatch(self):
        self.combinations = self.combinatronics_of_params()
        self.urls = self.concat_urls()
        self.urls_to_filter = self.get_image_urls()
        return set(self.match_pattern())

    def match_pattern(self):
        clean_urls = []
        for url in self.urls_to_filter:
            if (not PATTERNS['gallery'](url)) and \
                    (not PATTERNS['comments'](url)):
                if (PATTERNS['ext'](url)):
                    clean_urls.append(url)
                elif not PATTERNS['not_img'](url):
                    merged = url + '.jpg'
                    clean_urls.append(merged)
            elif PATTERNS['gallery'](url):
                gallery = ImgurGallery(**{'url': url})
                gallery_list = gallery.dispatch()
                clean_urls.extend(gallery_list)


        return clean_urls

    def extract_image_url(self, data):
        links = []
        for child in data['data']['children']:
            score = child['data']['score']
            if int(score) >= int(self.p):
                links.append(child['data']['url'])
        return links

    def get_json(self, url):
        r = requests.get(url, headers=request_headers)
        # r = requests.get(url)
        return r.json()

    def get_image_urls(self):
        image_urls = []
        count = 0
        wait = 3
        for url in self.urls:
            data = self.get_json(url)
            count += 1
            try:
                start = time.time() + wait
                time.sleep(wait)
                image_urls += self.extract_image_url(data)
                elapsed = start - time.time()
            except KeyError as error:
                eprint('error \n')
                eprint('url: ', url)
                eprint(data)
        return image_urls

    def form_urls(self, index, subreddit, order, count):
        url = r'http://www.reddit.com/' + index + '/' + subreddit
        url += '/' + order

        tail = '/.json' + '?limit=' + count
        url += tail
        return url

    def concat_urls(self):
        urls = []
        for item in self.combinations:
            urls.append(self.form_urls(*item))
        return urls

    def combinatronics_of_params(self):
        clean_subreddits = [x.strip() for x in self.s.split(',')]
        clean_subreddits = filter(None, clean_subreddits)
        clean_order = [x.strip() for x in self.o.split(',')]
        clean_order = filter(None, clean_order)
        return list(itertools.product(
                    self.i,
                    clean_subreddits,
                    clean_order,
                    [self.c, ]))


class ImgurGallery:

    page = None

    def __init__(self, *args, **kwargs):
        self.__dict__.update(kwargs)

    def dispatch(self):
        self.get_page()
        return self.get_gallery()

    def get_page(self):
        try:
            self.page = requests.get(self.url, headers=request_headers)
        except:
            raise Exception('spam', 'eggs')
        return

    def get_gallery(self):
        soup = BeautifulSoup(self.page.content, 'html.parser')
        container = soup.find('div', {'class': 'post-images'})
        if container:
            a_src = ['http:' + img.get('src') for img in container.find_all('img')]
            return a_src
        return []



import os
from setuptools import setup, find_packages

CURRENT_DIR = os.path.dirname(__file__)
def read(fname):
    return open(os.path.join(CURRENT_DIR, fname)).read()

# Info for setup
PACKAGE = 'reddit_view'
NAME = 'reddit_view'
DESCRIPTION = 'a reddit image collector'
AUTHOR = 'Jorge Perez'
AUTHOR_EMAIL = 'japrogramer@gmail.com'
URL = 'https://github.com/japrogramer/reddit_view'
VERSION = __import__(PACKAGE).__version__

# setup call
setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=read('README.rst'),
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license='BSD',
    url=URL,
    packages=find_packages(exclude=["tests.*", "tests"]),
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
         'Environment :: Web Environment',
         'Intended Audience :: Developers',
         'License :: OSI Approved :: BSD License',
         'Operating System :: OS Independent',
         'Programming Language :: Python',
    ],
    keywords = 'reddit images imgur links list subreddits',
    install_requires=[
         'requests',
         'beautifulsoup4',
         'fake-useragent',
    ],
    zip_safe=False,
    entry_points={
        'console_scripts': ['reddit_view=reddit_view.__main__:main',],},
    )
